@@ -28,7 +28,14 @@ from decimal import Decimal
 from weka import arff
 from weka.arff import SPARSE, DENSE, Num, Nom, Int, Str, Date
 
+log_level_id = getattr(logging, os.environ.get('LOGLEVEL', 'INFO'))
 logger = logging.getLogger(__name__)
+logger.setLevel(log_level_id)
+handler = logging.StreamHandler()
+handler.setLevel(log_level_id)
+formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(message)s", "%Y-%m-%d %H:%M:%S")
+handler.setFormatter(formatter)
+logger.addHandler(handler)
 
 DEFAULT_WEKA_JAR_PATH = '/usr/share/java/weka.jar:/usr/share/java/libsvm.jar'
 
@@ -169,6 +176,9 @@ class PredictionResult:
     def __str__(self):
         return f'<{type(self).__name__}: actual={self.actual}, predicted={self.predicted}, probability={self.probability}>'
 
+    def __repr__(self):
+        return str(self)
+
     @property
     def certainty(self):
         return self.probability.get(self.predicted)
@@ -280,8 +290,15 @@ class Classifier(BaseClassifier):
         ckargs = []
         if self.ckargs:
             for k, v in self.ckargs.items():
+
+                # Ensure each top-level arg has a "-" prefix.
                 if not k.startswith('-'):
                     k = '-' + k
+
+                # Quote arg value if it contains spaces.
+                if isinstance(v, str) and ' ' in v:
+                    v = f'"{v}"'
+
                 if v is None:
                     ckargs.append(f'{k}')
                 else:
@@ -377,10 +394,10 @@ class Classifier(BaseClassifier):
             self.last_training_stderr = stderr_str
 
             if verbose:
-                print('stdout:')
-                print(stdout_str)
-                print('stderr:')
-                print(stderr_str)
+                print('train.stdout:')
+                print(stdout_str.decode('utf-8'))
+                print('train.stderr:')
+                print(stderr_str.decode('utf-8'))
             # exclude "Warning" lines not to raise an error for a simple warning
             stderr_str = '\n'.join(l for l in stderr_str.decode('utf8').split('\n') if not "Warning" in l)
             if stderr_str:
@@ -445,13 +462,6 @@ class Classifier(BaseClassifier):
 
             # Call Weka Jar.
             distribution = ('-distribution' if distribution else '')
-            # args = dict(
-            # CP=CP,
-            # classifier_name=self.name,
-            # model_fn=model_fn,
-            # query_fn=query_fn,
-            # distribution=('-distribution' if distribution else ''),
-            # )
             cmd = f"java -cp {CP} {self.name} -p 0 {distribution} -l \"{model_fn}\" -T \"{query_fn}\""
             logger.debug('Querying predictor: %s', cmd)
             p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE, close_fds=True) # pylint: disable=consider-using-with
@@ -459,24 +469,19 @@ class Classifier(BaseClassifier):
             stdout_str = stdout.read()
             stderr_str = stderr.read()
             if verbose:
-                print('stdout:')
-                print(stdout_str)
-                print('stderr:')
-                print(stderr_str)
+                print('predict.stdout:')
+                print(stdout_str.decode('utf-8'))
+                print('predict.stderr:')
+                print(stderr_str.decode('utf-8'))
             if stderr_str:
                 raise PredictionError(stderr_str)
 
             if stdout_str:
-                # inst#     actual  predicted error prediction
-                #header = 'inst,actual,predicted,error'.split(',')
                 query = arff.ArffFile.load(query_fn)
                 query_variables = [query.attributes[i] for i, v in enumerate(query.data[0]) if v == arff.MISSING]
                 if not query_variables:
                     query_variables = [query.attributes[-1]]
 
-
-#                assert query_variables, \
-#                    "There must be at least one query variable in the query."
                 if verbose:
                     print('query_variables:', query_variables)
                 header = 'predicted'.split(',')
